package com.supervision.ai.service.hub.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.TimeInterval;
import cn.hutool.core.lang.UUID;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.supervision.ai.service.hub.dto.heygem.*;
import com.supervision.ai.service.hub.service.HeygemApiService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class HeygemApiServiceImpl implements HeygemApiService {

    @Value("${heygem.server.ip}")
    private String heygemServerIp;

    @Value("${heygem.tts.port}")
    private String heygemTtsPort;

    @Value("${heygem.f2f.port}")
    private String heygemF2fPort;

    private static final ObjectMapper objectMapper = new ObjectMapper();

    @Override
    public TranResDTO preprocessAndTran(TranReqDTO tranReqDTO) {
        String url = "http://" + heygemServerIp + ":" + heygemTtsPort + "/v1/preprocess_and_tran";
        log.info("url:{}", url);
        TimeInterval timer = DateUtil.timer();
        HttpRequest request = HttpUtil.createPost(url).body(JSONUtil.toJsonStr(tranReqDTO));
        try (HttpResponse response = request.execute()) {
            log.info("/v1/preprocess_and_tran请求耗时:{}", timer.intervalMs());
            if (!response.isOk()){
                throw new RuntimeException("请求失败");
            }
            String body = response.body();
            log.info("/v1/preprocess_and_tran接口response:{}", body);
            try {
                return objectMapper.readValue(body, new TypeReference<>() {});
            } catch (JsonProcessingException e) {
                throw new RuntimeException(e);
            }
        }
    }

    @Override
    public TextToAudioResDTO textToAudio(TextToAudioReqDTO textToAudioReqDTO) {
        String url = "http://" + heygemServerIp + ":" + heygemTtsPort + "/v1/invoke";
        log.info("url:{}", url);
        HttpRequest request = HttpUtil.createPost(url).body(JSONUtil.toJsonStr(textToAudioReqDTO));
        TimeInterval timer = DateUtil.timer();
        try (HttpResponse response = request.execute()) {
            log.info("/v1/invoke请求耗时:{}", timer.intervalMs());
            if (!response.isOk()&& response.header("Content-Type").equals("application/json")){
                try {
                    String body = response.body();
                    log.info("/v1/invoke接口response:{}", body);
                    return objectMapper.readValue(response.body(), new TypeReference<>() {});
                } catch (JsonProcessingException e) {
                    throw new RuntimeException(e);
                }
            }
            if (response.isOk()){
                byte[] bodyBytes = response.bodyBytes();
                TextToAudioResDTO textToAudioResDTO = new TextToAudioResDTO();
                textToAudioResDTO.setStatusCode("0");
                textToAudioResDTO.setAudio(bodyBytes);
                return textToAudioResDTO;
            }
            throw new RuntimeException("请求失败");
        }
    }
    @Override
    public Response<Object> submitVideoGenerateTask(VideoGenerateReqDTO videoGenerateReqDTO) {

        String url = "http://" + heygemServerIp + ":" + heygemF2fPort + "/easy/submit";
        log.info("url:{}", url);
        videoGenerateReqDTO.setCode(UUID.fastUUID().toString());
        HttpRequest request = HttpUtil.createPost(url).body(JSONUtil.toJsonStr(videoGenerateReqDTO));
        TimeInterval timer = DateUtil.timer();
        try (HttpResponse response = request.execute()) {
            log.info("请求/easy/submit耗时:{}", timer.intervalMs());
            String body = response.body();
            log.info("/easy/submit接口response:{}", body);
            try {
                Response<Object> readValue = objectMapper.readValue(body, new TypeReference<>() {});
                if (readValue.getCode()==10000){
                    readValue.setMsg(videoGenerateReqDTO.getCode());
                }
                return readValue;
            } catch (JsonProcessingException e) {
                throw new RuntimeException(e);
            }
        }
    }

    @Override
    public Response<TaskStatusResDTO> queryTaskStatus(String taskCode) {

        String url = "http://" + heygemServerIp + ":" + heygemF2fPort + "/easy/query";
        log.info("url:{}", url);
        TimeInterval timer = DateUtil.timer();
        try (HttpResponse response = HttpUtil.createGet(url).form("taskCode", taskCode).execute()) {
            log.info("请求/easy/query耗时:{}", timer.intervalMs());
            String body = response.body();
            log.info("/easy/query接口response:{}", body);
            try {
                return objectMapper.readValue(body, new TypeReference<>() {});
            } catch (JsonProcessingException e) {
                throw new RuntimeException(e);
            }
        }
    }
    
}
